@@ -8,7 +8,7 @@
 """
 
 from ip4tc import Table, Chain, Rule, Match, Target, Policy, IPTCError
-from ip6tc import Table6, Rule6
+from ip6tc import is_table_available, Table6, Rule6
 from xtables import XTablesError
 
 __all__ = []
