@@ -1,16 +1,12 @@
 # -*- coding: utf-8 -*-
 
-import os
-import re
 import ctypes as ct
 import socket
-import struct
 import weakref
-import ctypes.util
 
-from ip4tc import Rule, Table
-from xtables import XT_INV_PROTO, NFPROTO_IPV6, XTF_TRY_LOAD, XTablesError, xtables, xt_align, xt_counters, xt_entry_target, xt_entry_match
-from util import load_kernel
+from ip4tc import Rule, Table, IPTCError
+from util import find_library, load_kernel
+from xtables import (XT_INV_PROTO, NFPROTO_IPV6, xt_align, xt_counters)
 
 __all__ = ["Table6", "Rule6"]
 
@@ -18,58 +14,75 @@ load_kernel("ip6_tables")
 
 _IFNAMSIZ = 16
 
+
+def is_table_available(name):
+    try:
+        Table(name)
+        return True
+    except IPTCError:
+        pass
+    try:
+        Table6(name)
+        return True
+    except IPTCError:
+        pass
+    return False
+
+
 class in6_addr(ct.Structure):
     """This class is a representation of the C struct in6_addr."""
-    _fields_ = [("s6_addr", ct.c_uint8 * 16)] # IPv6 address
+    _fields_ = [("s6_addr", ct.c_uint8 * 16)]  # IPv6 address
+
 
 class ip6t_ip6(ct.Structure):
     """This class is a representation of the C struct ip6t_ip6."""
     _fields_ = [("src", in6_addr),  # Source and destination IP6 addr
-          ("dst", in6_addr),        # Mask for src and dest IP6 addr
-          ("smsk", in6_addr),
-          ("dmsk", in6_addr),
-          ("iniface", ct.c_char * _IFNAMSIZ),
-          ("outiface", ct.c_char * _IFNAMSIZ),
-          ("iniface_mask", ct.c_char * _IFNAMSIZ),
-          ("outiface_mask", ct.c_char * _IFNAMSIZ),
-          ("proto", ct.c_uint16),   # Upper protocol number
-          ("tos", ct.c_uint8),      # TOS to match iff flags & IP6T_F_TOS
-          ("flags", ct.c_uint8),    # Flags word
-          ("invflags", ct.c_uint8)] # Inverse flags
+                ("dst", in6_addr),  # Mask for src and dest IP6 addr
+                ("smsk", in6_addr),
+                ("dmsk", in6_addr),
+                ("iniface", ct.c_char * _IFNAMSIZ),
+                ("outiface", ct.c_char * _IFNAMSIZ),
+                ("iniface_mask", ct.c_char * _IFNAMSIZ),
+                ("outiface_mask", ct.c_char * _IFNAMSIZ),
+                ("proto", ct.c_uint16),    # Upper protocol number
+                ("tos", ct.c_uint8),       # TOS, match iff flags & IP6T_F_TOS
+                ("flags", ct.c_uint8),     # Flags word
+                ("invflags", ct.c_uint8)]  # Inverse flags
 
     # flags
     IP6T_F_PROTO = 0x01    # Set if rule cares about upper protocols
-    IP6T_F_TOS   = 0x02    # Match the TOS
-    IP6T_F_GOTO  = 0x04    # Set if jump is a goto
-    IP6T_F_MASK  = 0x07    # All possible flag bits mask
+    IP6T_F_TOS = 0x02    # Match the TOS
+    IP6T_F_GOTO = 0x04    # Set if jump is a goto
+    IP6T_F_MASK = 0x07    # All possible flag bits mask
 
     # invflags
-    IP6T_INV_VIA_IN  = 0x01    # Invert the sense of IN IFACE
+    IP6T_INV_VIA_IN = 0x01    # Invert the sense of IN IFACE
     IP6T_INV_VIA_OUT = 0x02    # Invert the sense of OUT IFACE
-    IP6T_INV_TOS     = 0x04    # Invert the sense of TOS
-    IP6T_INV_SRCIP   = 0x08    # Invert the sense of SRC IP
-    IP6T_INV_DSTIP   = 0x10    # Invert the sense of DST OP
-    IP6T_INV_FRAG    = 0x20    # Invert the sense of FRAG
-    IP6T_INV_PROTO   = XT_INV_PROTO
-    IP6T_INV_MASK    = 0x7F    # All possible flag bits mask
+    IP6T_INV_TOS = 0x04    # Invert the sense of TOS
+    IP6T_INV_SRCIP = 0x08    # Invert the sense of SRC IP
+    IP6T_INV_DSTIP = 0x10    # Invert the sense of DST OP
+    IP6T_INV_FRAG = 0x20    # Invert the sense of FRAG
+    IP6T_INV_PROTO = XT_INV_PROTO
+    IP6T_INV_MASK = 0x7F    # All possible flag bits mask
 
     def __init__(self):
-        self.smsk.s6_addr = self.dmsk.s6_addr = 0xff * 16 # default: full netmask
+        # default: full netmask
+        self.smsk.s6_addr = self.dmsk.s6_addr = 0xff * 16
+
 
 class ip6t_entry(ct.Structure):
     """This class is a representation of the C struct ip6t_entry."""
     _fields_ = [("ipv6", ip6t_ip6),
-          ("nfcache", ct.c_uint),         # Mark with fields that we care about
-          ("target_offset", ct.c_uint16), # Size of ip6t_entry + matches
-          ("next_offset", ct.c_uint16),   # Size of ip6t_entry + matches + target
-          ("comefrom", ct.c_uint),        # Back pointer
-          ("counters", xt_counters),      # Packet and byte counters
-          ("elems", ct.c_ubyte * 0)]      # The matches (if any) then the target
+                ("nfcache", ct.c_uint),          # fields that we care about
+                ("target_offset", ct.c_uint16),  # size of ip6t_entry + matches
+                ("next_offset", ct.c_uint16),    # size of e + matches + target
+                ("comefrom", ct.c_uint),         # back pointer
+                ("counters", xt_counters),       # packet and byte counters
+                ("elems", ct.c_ubyte * 0)]       # the matches then the target
+
+
+_libiptc, _ = find_library("ip6tc", "iptc")  # old iptables versions use iptc
 
-_libiptc_file = ctypes.util.find_library("ip6tc")
-if not _libiptc_file:
-    raise IPTCError("error: libip6tc not found")
-_libiptc = ct.CDLL(_libiptc_file, use_errno = True)
 
 class ip6tc(object):
     """This class contains all libip6tc API calls."""
@@ -125,13 +138,13 @@ class ip6tc(object):
     iptc_get_policy = _libiptc.ip6tc_get_policy
     iptc_get_policy.restype = ct.c_char_p
     iptc_get_policy.argstype = [ct.c_char_p, ct.POINTER(xt_counters),
-          ct.c_void_p]
+                                ct.c_void_p]
 
     # Set the policy of a chain
     iptc_set_policy = _libiptc.ip6tc_set_policy
     iptc_set_policy.restype = ct.c_int
     iptc_set_policy.argstype = [ct.c_char_p, ct.c_char_p,
-          ct.POINTER(xt_counters), ct.c_void_p]
+                                ct.POINTER(xt_counters), ct.c_void_p]
 
     # Get first rule in the given chain: NULL for empty chain.
     iptc_first_rule = _libiptc.ip6tc_first_rule
@@ -155,28 +168,28 @@ class ip6tc(object):
     # Insert the entry `e' in chain `chain' into position `rulenum'.
     iptc_insert_entry = _libiptc.ip6tc_insert_entry
     iptc_insert_entry.restype = ct.c_int
-    iptc_insert_entry.argstype = [ct.c_char_p, ct.POINTER(ip6t_entry), ct.c_int,
-          ct.c_void_p]
+    iptc_insert_entry.argstype = [ct.c_char_p, ct.POINTER(ip6t_entry),
+                                  ct.c_int, ct.c_void_p]
 
     # Atomically replace rule `rulenum' in `chain' with `e'.
     iptc_replace_entry = _libiptc.ip6tc_replace_entry
     iptc_replace_entry.restype = ct.c_int
-    iptc_replace_entry.argstype = [ct.c_char_p, ct.POINTER(ip6t_entry), ct.c_int,
-          ct.c_void_p]
+    iptc_replace_entry.argstype = [ct.c_char_p, ct.POINTER(ip6t_entry),
+                                   ct.c_int, ct.c_void_p]
 
     # Append entry `e' to chain `chain'.  Equivalent to insert with
     #   rulenum = length of chain.
     iptc_append_entry = _libiptc.ip6tc_append_entry
     iptc_append_entry.restype = ct.c_int
     iptc_append_entry.argstype = [ct.c_char_p, ct.POINTER(ip6t_entry),
-            ct.c_void_p]
+                                  ct.c_void_p]
 
     # Delete the first rule in `chain' which matches `e', subject to
     #   matchmask (array of length == origfw)
     iptc_delete_entry = _libiptc.ip6tc_delete_entry
     iptc_delete_entry.restype = ct.c_int
     iptc_delete_entry.argstype = [ct.c_char_p, ct.POINTER(ip6t_entry),
-          ct.POINTER(ct.c_ubyte), ct.c_void_p]
+                                  ct.POINTER(ct.c_ubyte), ct.c_void_p]
 
     # Delete the rule in position `rulenum' in `chain'.
     iptc_delete_num_entry = _libiptc.ip6tc_delete_num_entry
@@ -208,13 +221,14 @@ class ip6tc(object):
     iptc_set_counter = _libiptc.ip6tc_set_counter
     iptc_set_counter.restype = ct.c_int
     iptc_set_counter.argstype = [ct.c_char_p, ct.c_uint,
-          ct.POINTER(xt_counters), ct.c_void_p]
+                                 ct.POINTER(xt_counters), ct.c_void_p]
 
     # Translates errno numbers into more human-readable form than strerror.
     iptc_strerror = _libiptc.ip6tc_strerror
     iptc_strerror.restype = ct.c_char_p
     iptc_strerror.argstype = [ct.c_int]
 
+
 class Rule6(Rule):
     """This is an IPv6 rule."""
 
@@ -230,13 +244,13 @@ class Rule6(Rule):
             return False
         if len(self._matches) != len(rule._matches):
             return False
-        if set(rule._matches) != set([x for x in rule._matches if x in
-                self._matches]):
+        if set(rule._matches) != set([x for x in rule._matches
+                                      if x in self._matches]):
             return False
-        if self.src == rule.src and self.dst == rule.dst and \
-                self.protocol == rule.protocol and \
-                self.in_interface == rule.in_interface and \
-                self.out_interface == rule.out_interface:
+        if (self.src == rule.src and self.dst == rule.dst and
+            self.protocol == rule.protocol and
+            self.in_interface == rule.in_interface and
+            self.out_interface == rule.out_interface):
             return True
         return False
 
@@ -244,7 +258,7 @@ class Rule6(Rule):
         return self._save(name, self.entry.ipv6)
 
     def _get_tables(self):
-        return [Table6(t) for t in Table6.ALL]
+        return [Table6(t) for t in Table6.ALL if is_table_available(t)]
     tables = property(_get_tables)
     """This is the list of tables for our protocol."""
 
@@ -274,8 +288,8 @@ class Rule6(Rule):
             src = "".join([src, "!"])
         try:
             addr = socket.inet_ntop(socket.AF_INET6,
-                    self.entry.ipv6.src.s6_addr)
-        except socket.error as e:
+                                    self.entry.ipv6.src.s6_addr)
+        except socket.error:
             raise IPTCError("error in internal state: invalid address")
         src = "".join([src, addr, "/"])
 
@@ -295,8 +309,8 @@ class Rule6(Rule):
             self.entry.ipv6.invflags |= ip6t_ip6.IP6T_INV_SRCIP
             src = src[1:]
         else:
-            self.entry.ipv6.invflags &= ~ip6t_ip6.IP6T_INV_SRCIP & \
-                  ip6t_ip6.IP6T_INV_MASK
+            self.entry.ipv6.invflags &= (~ip6t_ip6.IP6T_INV_SRCIP &
+                                         ip6t_ip6.IP6T_INV_MASK)
 
         slash = src.find("/")
         if slash == -1:
@@ -310,8 +324,8 @@ class Rule6(Rule):
         ina = in6_addr()
         try:
             ina.s6_addr = arr.from_buffer_copy(
-                    socket.inet_pton(socket.AF_INET6, addr))
-        except socket.error as e:
+                socket.inet_pton(socket.AF_INET6, addr))
+        except socket.error:
             raise ValueError("invalid address %s" % (addr))
         self.entry.ipv6.src = ina
 
@@ -321,15 +335,15 @@ class Rule6(Rule):
             if plen < 0 or plen > 128:
                 raise ValueError("invalid prefix length %d" % (plen))
             self.entry.ipv6.smsk.s6_addr = arr.from_buffer_copy(
-                    self._create_mask(plen))
+                self._create_mask(plen))
             return
 
         # nope, we got an IPv6 address-style prefix
         neta = in6_addr()
         try:
             neta.s6_addr = arr.from_buffer_copy(
-                    socket.inet_pton(socket.AF_INET6, netm))
-        except socket.error as e:
+                socket.inet_pton(socket.AF_INET6, netm))
+        except socket.error:
             raise ValueError("invalid netmask %s" % (netm))
         self.entry.ipv6.smsk = neta
 
@@ -343,8 +357,8 @@ class Rule6(Rule):
             dst = "".join([dst, "!"])
         try:
             addr = socket.inet_ntop(socket.AF_INET6,
-                    self.entry.ipv6.dst.s6_addr)
-        except socket.error as e:
+                                    self.entry.ipv6.dst.s6_addr)
+        except socket.error:
             raise IPTCError("error in internal state: invalid address")
         dst = "".join([dst, addr, "/"])
 
@@ -364,8 +378,8 @@ class Rule6(Rule):
             self.entry.ipv6.invflags |= ip6t_ip6.IP6T_INV_DSTIP
             dst = dst[1:]
         else:
-            self.entry.ipv6.invflags &= ~ip6t_ip6.IP6T_INV_DSTIP & \
-                  ip6t_ip6.IP6T_INV_MASK
+            self.entry.ipv6.invflags &= (~ip6t_ip6.IP6T_INV_DSTIP &
+                                         ip6t_ip6.IP6T_INV_MASK)
 
         slash = dst.find("/")
         if slash == -1:
@@ -379,8 +393,8 @@ class Rule6(Rule):
         ina = in6_addr()
         try:
             ina.s6_addr = arr.from_buffer_copy(
-                    socket.inet_pton(socket.AF_INET6, addr))
-        except socket.error as e:
+                socket.inet_pton(socket.AF_INET6, addr))
+        except socket.error:
             raise ValueError("invalid address %s" % (addr))
         self.entry.ipv6.dst = ina
 
@@ -390,15 +404,15 @@ class Rule6(Rule):
             if plen < 0 or plen > 128:
                 raise ValueError("invalid prefix length %d" % (plen))
             self.entry.ipv6.dmsk.s6_addr = arr.from_buffer_copy(
-                    self._create_mask(plen))
+                self._create_mask(plen))
             return
 
         # nope, we got an IPv6 address-style prefix
         neta = in6_addr()
         try:
             neta.s6_addr = arr.from_buffer_copy(
-                    socket.inet_pton(socket.AF_INET6, netm))
-        except socket.error as e:
+                socket.inet_pton(socket.AF_INET6, netm))
+        except socket.error:
             raise ValueError("invalid netmask %s" % (netm))
         self.entry.ipv6.dmsk = neta
 
@@ -432,8 +446,8 @@ class Rule6(Rule):
             self.entry.ipv6.invflags |= ip6t_ip6.IP6T_INV_VIA_IN
             intf = intf[1:]
         else:
-            self.entry.ipv6.invflags &= ~ip6t_ip6.IP6T_INV_VIA_IN & \
-                  ip6t_ip6.IP6T_INV_MASK
+            self.entry.ipv6.invflags &= (~ip6t_ip6.IP6T_INV_VIA_IN &
+                                         ip6t_ip6.IP6T_INV_MASK)
         if len(intf) >= _IFNAMSIZ:
             raise ValueError("interface name %s too long" % (intf))
         masklen = len(intf) + 1
@@ -441,10 +455,10 @@ class Rule6(Rule):
             intf = intf[:-1]
             masklen -= 2
 
-        self.entry.ipv6.iniface = \
-              "".join([intf, '\x00' * (_IFNAMSIZ - len(intf))])
-        self.entry.ipv6.iniface_mask = \
-              "".join(['\x01' * masklen, '\x00' * (_IFNAMSIZ - masklen)])
+        self.entry.ipv6.iniface = "".join([intf, '\x00' * (_IFNAMSIZ -
+                                                           len(intf))])
+        self.entry.ipv6.iniface_mask = "".join(['\x01' * masklen, '\x00' *
+                                                (_IFNAMSIZ - masklen)])
 
     in_interface = property(get_in_interface, set_in_interface)
     """This is the input network interface e.g. *eth0*.  A wildcard match can
@@ -455,11 +469,10 @@ class Rule6(Rule):
         if self.entry.ipv6.invflags & ip6t_ip6.IP6T_INV_VIA_OUT:
             intf = "".join(["!", intf])
         iface = bytearray(_IFNAMSIZ)
-        iface[:len(self.entry.ipv6.outiface)] = \
-              self.entry.ipv6.outiface
+        iface[:len(self.entry.ipv6.outiface)] = self.entry.ipv6.outiface
         mask = bytearray(_IFNAMSIZ)
         mask[:len(self.entry.ipv6.outiface_mask)] = \
-              self.entry.ipv6.outiface_mask
+            self.entry.ipv6.outiface_mask
         if mask[0] == 0:
             return None
         for i in xrange(_IFNAMSIZ):
@@ -478,8 +491,8 @@ class Rule6(Rule):
             self.entry.ipv6.invflags |= ip6t_ip6.IP6T_INV_VIA_OUT
             intf = intf[1:]
         else:
-            self.entry.ipv6.invflags &= ~ip6t_ip6.IP6T_INV_VIA_OUT & \
-                  ip6t_ip6.IP6T_INV_MASK
+            self.entry.ipv6.invflags &= (~ip6t_ip6.IP6T_INV_VIA_OUT &
+                                         ip6t_ip6.IP6T_INV_MASK)
         if len(intf) >= _IFNAMSIZ:
             raise ValueError("interface name %s too long" % (intf))
         masklen = len(intf) + 1
@@ -487,10 +500,10 @@ class Rule6(Rule):
             intf = intf[:-1]
             masklen -= 2
 
-        self.entry.ipv6.outiface = \
-              "".join([intf, '\x00' * (_IFNAMSIZ - len(intf))])
-        self.entry.ipv6.outiface_mask = \
-              "".join(['\x01' * masklen, '\x00' * (_IFNAMSIZ - masklen)])
+        self.entry.ipv6.outiface = "".join([intf, '\x00' * (_IFNAMSIZ -
+                                                            len(intf))])
+        self.entry.ipv6.outiface_mask = "".join(['\x01' * masklen, '\x00' *
+                                                 (_IFNAMSIZ - masklen)])
 
     out_interface = property(get_out_interface, set_out_interface)
     """This is the output network interface e.g. *eth0*.  A wildcard match can
@@ -509,8 +522,8 @@ class Rule6(Rule):
             self.entry.ipv6.invflags |= ip6t_ip6.IP6T_INV_PROTO
             proto = proto[1:]
         else:
-            self.entry.ipv6.invflags &= \
-                  ~ip6t_ip6.IP6T_INV_PROTO & ip6t_ip6.IP6T_INV_MASK
+            self.entry.ipv6.invflags &= (~ip6t_ip6.IP6T_INV_PROTO &
+                                         ip6t_ip6.IP6T_INV_MASK)
         for p in self.protocols.items():
             if proto.lower() == p[1]:
                 self.entry.ipv6.proto = p[0]
@@ -532,6 +545,7 @@ class Rule6(Rule):
     def _new_entry(self):
         return ip6t_entry()
 
+
 class Table6(Table):
     """The IPv6 version of Table.
 
@@ -567,7 +581,7 @@ class Table6(Table):
 
     _cache = weakref.WeakValueDictionary()
 
-    def __new__(cls, name, autocommit = True):
+    def __new__(cls, name, autocommit=True):
         obj = Table6._cache.get(name, None)
         if not obj:
             obj = object.__new__(cls)
@@ -576,15 +590,18 @@ class Table6(Table):
             obj.autocommit = autocommit
         return obj
 
-    def __init__(self, name, autocommit = True):
+    def __init__(self, name, autocommit=True):
         """
         Here *name* is the name of the table to instantiate, if it has already
         been instantiated the existing cached object is returned.
         *Autocommit* specifies that any low-level iptables operation should be
         committed immediately, making changes visible in the kernel.
         """
-        self._iptc = ip6tc() # to keep references to functions
+        self._iptc = ip6tc()  # to keep references to functions
         self._handle = None
         self.name = name
         self.autocommit = autocommit
         self.refresh()
+
+    def create_rule(self, entry=None, chain=None):
+        return Rule6(entry, chain)
